function Scatter = ssm_from_qsm( x, seed )
%SSM_FROM_QSM generates the Scatter {B2,S1} from the Espoo maple QSM with some
%little noise added to the QSM data set.
% Usage:
%       Scatter = ssm_from_qsm( x )
% x: vector of parameters. Dim: 4x1:
%   x(1) - mean displacement amplitude of B2 scatter
%   x(2) - amplitude of noise for B2 scatter
%   x(3) - mean displacement amplitude of S1 scatter
%   x(4) - amplitude of noise for S1 scatter
% Scatter: the output scatter

% Fix the RNG seed to any number to reproduce the result during
% optimization
rng(56);
if nargin == 2
    rng(seed);
end

% Generate scatter
Scatter = cell(1,2);
% Read the QSM data first
Scatter{1} = dlmread('test2-B2.dat');
Scatter{2} = dlmread('test2-S1.dat');
% Calculate the mean of the variates
m1 = mean(Scatter{1},2);
m2 = mean(Scatter{2},2);
% Finally add Gauss noise (x(2) and x(4)) and add displacement in terms of
% means (x(1) and x(3))
Scatter{1} = Scatter{1} + x(1).*repmat(m1,1,size(Scatter{1},2)) + x(2).*randn(size(Scatter{1}));
Scatter{2} = Scatter{2} + x(3).*repmat(m2,1,size(Scatter{2},2)) + x(4).*randn(size(Scatter{2}));


end

